// Set the default precision to medium. We don't need as high of a precision in the fragment shader.
precision mediump float;

const float lf_deviation = 0.1;  // low frequency deviation
const float hf_deviation = 0.05; // high frequency deviation

const float lf_step = 0.5;  // low frequency step
const float hf_step = 0.05; // high frequency step

const float PI = 3.14159265;

const float low_frequency = 2.0 * PI / lf_step;
const float high_frequency = 2.0 * PI / hf_step;

const float lf_min = 0.99; // low frequency min border
const float lf_max = 1.0;  // low frequency max border

const float hf_min = 0.94; // high frequency min border
const float hf_max = 1.0;  // high frequency max border

// This is the color from the vertex shader interpolated across the triangle per fragment.
varying vec3 v_Position;
varying vec3 v_Normal;
varying vec4 v_Color;
varying vec3 v_StartPoint;
varying vec4 v_AllColor;

varying vec2 v_ScreenSize;
varying float v_Time;
varying vec3 v_IntersectionPoint;

void main() {

    float y = v_Position.y - v_StartPoint.y;
    vec3 xz = vec3(v_Position.x - v_StartPoint.x, 0.0, v_Position.z - v_StartPoint.z);
//    vec3 intersectionVectorXZ = vec3(v_Position.x - v_IntersectionPoint.x, 0.0, v_Position.z - v_IntersectionPoint.z);

    //    vec3 perpNormal = vec3(v_Normal.z, 0, -v_Normal.x);
    //    float proj = dot(xz, perpNormal);
    float proj = length(xz);

    float lf_xz = cos(proj * low_frequency);
    float lf_y = cos(y * low_frequency);
    float hf_xz = cos(proj * high_frequency);
    float hf_y = cos(y * high_frequency);

    float alpha = 0.0;
    if (lf_xz > lf_min) {
        alpha += 0.8 * lf_xz;
//        alpha += 0.8 * smoothstep(lf_min, lf_max, lf_xz);
    }

    if (lf_y > lf_min) {
        alpha += 0.8 * lf_y;
//        alpha += 0.8 * smoothstep(lf_min, lf_max, lf_y);
    }

    if (hf_xz > hf_min) {
        alpha += 0.8 * hf_xz;
//        alpha += 0.8 * smoothstep(hf_min, hf_max, hf_xz);
    }

    if (hf_y > hf_min) {
        alpha += 0.8 * hf_y;
//        alpha += 0.8 * smoothstep(hf_min, hf_max, hf_y);
    }

    if (v_Position.y > v_IntersectionPoint.y) {
        alpha *= 0.5;
    }

    if (alpha > 1.0) alpha = 1.0;
    alpha *= v_AllColor.a;

//    alpha /= (0.04 * y * y + 1.0);

//    gl_FragColor = vec4(1.0, 1.0, 1.0, alpha);
    gl_FragColor = v_AllColor;
    gl_FragColor.a = alpha;
}

// smoothstep(edge0, edge1, x) :
// t = clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);
// return t * t * (3.0 - 2.0 * t);
