// Set the default precision to medium. We don't need as high of a precision in the fragment shader.
precision mediump float;

uniform sampler2D u_Texture;// The input texture.

// This is the color from the vertex shader interpolated across the triangle per fragment.
varying vec3 v_Position;
varying vec3 v_Normal;
varying vec4 v_Color;
varying vec2 v_TexCoordinate;

uniform vec3 u_LightPosition;
uniform vec3 u_LightDirection;

varying vec2 v_ScreenSize;
varying float v_Time;
varying vec2 v_TouchPoint;

varying mat4 v_MVMatrix;
uniform bool u_LightIsOn;

//const float kGamma = 0.4545454;
const float kGamma = 0.2;
const float kInverseGamma = 5.0; // 2.2
const float kMiddleGrayGamma = 0.466;
const float ambient = 0.2;
const float materialDiffuse = 1.0;
const float materialSpecular = 0.1;
const float specularPower = 3.0;

void main() {

    // Will be used for attenuation.
    // float distanceTo = length(u_LightPosition - v_Position);

    // Calculate the dot product of the light vector and vertex normal. If the normal and light vector are
    // pointing in the same direction then it will get max illumination.

    if (u_LightIsOn) {
        // We support approximate sRGB gamma.
        vec4 objectColor = v_Color;
        objectColor.rgb = pow(objectColor.rgb, vec3(kInverseGamma));
        vec3 viewFragmentDirection = normalize(v_Position);

        // Approximate a hemisphere light (not a harsh directional light).
        float diffuse = materialDiffuse * 0.5 * (dot(v_Normal, u_LightDirection) + 1.0);
        vec3 reflectedLightDirection = reflect(u_LightDirection, v_Normal);
        float specularStrength = max(0.0, dot(viewFragmentDirection, reflectedLightDirection));
        float specular = objectColor.a * materialSpecular * pow(specularStrength, specularPower);

        vec3 color = objectColor.rgb * (ambient + diffuse) + specular;
        // Apply SRGB gamma before writing the fragment color.
        color.rgb = pow(color, vec3(kGamma));
        // Apply average pixel intensity and color shift
        color *= 0.9;
        gl_FragColor.rgb = color;
        gl_FragColor.a = v_Color.a;
    } else {
        gl_FragColor = v_Color;
    }

    // Multiply the color by the diffuse illumination level to get final output color.
    //    gl_FragColor = (v_Color * diffuse);
    //    gl_FragColor = v_Color;
    //    gl_FragColor = (v_Color * diffuse * texture2D(u_Texture, v_TexCoordinate));
    //    gl_FragColor = (texture2D(u_Texture, v_TexCoordinate));

    //    vec4 color = (v_Color + texture2D(u_Texture, v_TexCoordinate));
    //        vec4 color = (texture2D(u_Texture, v_TexCoordinate));
    //    vec4 color = v_Color;
    //    vec4 color = vec4(0.25, 1.0, 0.0, 1.0);
    //    color.a = 0.7;

    //    float x = v_TexCoordinate.x - 0.5;
    //    float y = v_TexCoordinate.y - 0.5;
    //
    //    //    float r = sqrt(x * x + y * y);
    //    //
    //    //    if (r < 0.5) {
    //    //        color = vec4(1.0, 1.0, 0.0, 1.0);
    //    //    } else {
    //    //        color = vec4(0.0, 0.2, 0.2, 1.0);
    //    //    }
    //
    //    float sigma = 0.125;
    //    //    float f = 0.06 * time * 0.000001;
    //    float f = 0.06;
    //    //    float a = 0.2;
    //    float a = 0.2 - v_Time * f;
    //
    //    float r = exp(-(x * x + y * y) / (sigma));
    //    float g = cos(6.283 * (x * x + y * y) / f + a);
    //    float b = r * g;
    //
    //        color = vec4(b, b, b, 0.7);
    //
    //    float offset = 0.01;
    //
    //    if (x >= 0.0 && x < offset) {
    //        color *= 0.0;
    //        color.a = 1.0;
    //    }
    //
    //    if (x > 1.0 - offset && x <= 1.0) {
    //        color *= 0.0;
    //        color.a = 1.0;
    //    }
    //
    //    if (y >= 0.0 && y < offset) {
    //        color *= 0.0;
    //        color.a = 1.0;
    //    }
    //
    //    if (y > 1.0 - offset && y <= 1.0) {
    //        color *= 0.0;
    //        color.a = 1.0;
    //    }
    //
    //    if (x == 0.0 || x == 1.0 || y == 0.0 || y == 1.0) {
    //        color *= 0.0;
    //        color.a = 1.0;
    //    }

    //    gl_FragColor = color;
}

//
//// consider replacing this by a proper noise function
//float noise(in vec3 x) {
//    vec3 p = x;
//    vec3 f = x;
//    f = f * f * (3.0 - 2.0 * f);
//    vec2 uv = (p.xy + vec2(37.0, 17.0) * p.z) + f.xy;
//    vec2 rg = texture2D(u_Texture, (uv + 0.5) / 256.0, 0.0).yx;
//    return mix(rg.x, rg.y, f.z);
//}
//
//float mapTerrain(vec3 p) {
//    p *= 0.1;
//    p.xz *= 0.6;
//
//    float f;
//    f  = 0.500 * noise(p * 1.00);
//    f += 0.250 * noise(p * 1.00);
//    f += 0.125 * noise(p * 1.00);
//    return 25.0 * f-10.0;
//}
//
//vec3 gro = vec3(0.0);
//
//float map(in vec3 position) {
//    vec3 p = position + 0.5;
//    float f = mapTerrain(p) + 0.25*p.y;
//    f = mix(f, 1.0, step(length(gro-p), 5.0));
//    return step(f, 0.5);
//}
//
// vec3 lig = normalize(vec3(-0.4, 0.3, 0.7));
//
//float castRay(in vec3 rayOrigin, in vec3 rayDir, out vec3 oVos, out vec3 oDir) {
//    vec3 position = floor(rayOrigin);
//    vec3 ri = 1.0 / rayDir;
//    vec3 dis = (position - rayOrigin) * ri;
//
//    float res = -1.0;
//    vec3 mm = vec3(0.0);
//    for (int i = 0; i < 128; i++) {
//        if (map(position)>0.5) {
//            res = 1.0;
//            break;
//        }
//        mm = step(dis.xyz, dis.yzx) * step(dis.xyz, dis.zxy);
//        dis += mm * ri;
//        position += mm;
//    }
//
//    vec3 nor = -mm;
//    vec3 vos = position;
//
//    // intersect the cube
//    vec3 mini = (position - rayOrigin)*ri;
//    float t = max (mini.x, max (mini.y, mini.z));
//
//    oDir = mm;
//    oVos = vos;
//
//    return t * res;
//}
//
//float maxcomp(in vec4 v) {
//    return max(max(v.x, v.y), max(v.z, v.w));
//}
//
//float isEdge(in vec2 uv, vec4 va, vec4 vb, vec4 vc, vec4 vd) {
//    vec2 st = 1.0 - uv;
//
//    // edges
//    vec4 wb = smoothstep(0.85, 0.99, vec4(uv.x,
//    st.x,
//    uv.y,
//    st.y)) * (1.0 - va + va*vc);
//    // corners
//    vec4 wc = smoothstep(0.85, 0.99, vec4(uv.x*uv.y,
//    st.x*uv.y,
//    st.x*st.y,
//    uv.x*st.y)) * (1.0 - vb + vd*vb);
//    return maxcomp(max(wb, wc));
//}
//
//float calcOcclusion(in vec2 uv, vec4 va, vec4 vb, vec4 vc, vec4 vd) {
//    vec2 st = 1.0 - uv;
//
//    // edges
//    vec4 wa = vec4(uv.x, st.x, uv.y, st.y) * vc;
//
//    // corners
//    vec4 wb = vec4(uv.x * uv.y, st.x * uv.y, st.x * st.y, uv.x * st.y);
//    wb *= vd * (1.0 - vc.xzyw) * (1.0 - vc.zywx);
//    return (wa.x + wa.y + wa.z + wa.w) + (wb.x + wb.y + wb.z + wb.w);
//}
//
//vec3 render(in vec3 rayOrigin, in vec3 rayDir) {
//    vec3 color = vec3(0.0);
//
//    // raymarch
//    vec3 vos, dir;
//    float t = castRay(rayOrigin, rayDir, vos, dir);
//    if (t > 0.0) {
//        vec3 nor = -dir*sign(rayDir);
//        vec3 position = rayOrigin + rayDir*t;
//        vec3 uvw = position - vos;
//
//        vec3 v1  = vos + nor + dir.yzx;
//        vec3 v2  = vos + nor - dir.yzx;
//        vec3 v3  = vos + nor + dir.zxy;
//        vec3 v4  = vos + nor - dir.zxy;
//        vec3 v5  = vos + nor + dir.yzx + dir.zxy;
//        vec3 v6  = vos + nor - dir.yzx + dir.zxy;
//        vec3 v7  = vos + nor - dir.yzx - dir.zxy;
//        vec3 v8  = vos + nor + dir.yzx - dir.zxy;
//        vec3 v9  = vos + dir.yzx;
//        vec3 v10 = vos - dir.yzx;
//        vec3 v11 = vos + dir.zxy;
//        vec3 v12 = vos - dir.zxy;
//        vec3 v13 = vos + dir.yzx + dir.zxy;
//        vec3 v14 = vos - dir.yzx + dir.zxy;
//        vec3 v15 = vos - dir.yzx - dir.zxy;
//        vec3 v16 = vos + dir.yzx - dir.zxy;
//
//        vec4 vc = vec4(map(v1), map(v2), map(v3), map(v4));
//        vec4 vd = vec4(map(v5), map(v6), map(v7), map(v8));
//        vec4 va = vec4(map(v9), map(v10), map(v11), map(v12));
//        vec4 vb = vec4(map(v13), map(v14), map(v15), map(v16));
//
//        vec2 uv = vec2(dot(dir.yzx, uvw), dot(dir.zxy, uvw));
//
//        // wireframe
//        float wireframe_w = 1.0 - isEdge(uv, va, vb, vc, vd);
//
//        vec3 wireframe_wir = smoothstep(0.4, 0.5, abs(uvw-0.5));
//        float vvv = (1.0-wireframe_wir.x * wireframe_wir.y) * (1.0 - wireframe_wir.x*wireframe_wir.z)*(1.0-wireframe_wir.y*wireframe_wir.z);
//
//        color = vec3(0.5);
//        color += 0.8*vec3(0.1, 0.3, 0.4);
//        color *= 1.0 - 0.75*(1.0-vvv)*wireframe_w;
//
//        // lighting
//        float dif = clamp(dot(nor, lig), 0.0, 1.0);
//        float bac = clamp(dot(nor, normalize(lig*vec3(-1.0, 0.0, -1.0))), 0.0, 1.0);
//        float sky = 0.5 + 0.5*nor.y;
//        float amb = clamp(0.75 + position.y / 25.0, 0.0, 1.0);
//        float occlusion = 1.0;
//
//        // ambient occlusion
//        occlusion = calcOcclusion(uv, va, vb, vc, vd);
//        occlusion = 1.0 - occlusion/8.0;
//        occlusion = occlusion * occlusion;
//        occlusion = occlusion * occlusion;
//        occlusion *= amb;
//
//        // lighting
//        vec3 lin = vec3(0.0);
//        lin += 2.5*dif*vec3(1.00, 0.90, 0.70)*(0.5+0.5*occlusion);
//        lin += 0.5*bac*vec3(0.15, 0.10, 0.10)*occlusion;
//        lin += 2.0*sky*vec3(0.40, 0.30, 0.15)*occlusion;
//
//        // line glow
//        float lineglow = 0.0;
//        lineglow += smoothstep(0.4, 1.0, uv.x)*(1.0-va.x*(1.0-vc.x));
//        lineglow += smoothstep(0.4, 1.0, 1.0-uv.x)*(1.0-va.y*(1.0-vc.y));
//        lineglow += smoothstep(0.4, 1.0, uv.y)*(1.0-va.z*(1.0-vc.z));
//        lineglow += smoothstep(0.4, 1.0, 1.0-uv.y)*(1.0-va.w*(1.0-vc.w));
//        lineglow += smoothstep(0.4, 1.0, uv.y*      uv.x)*(1.0-vb.x*(1.0-vd.x));
//        lineglow += smoothstep(0.4, 1.0, uv.y* (1.0-uv.x))*(1.0-vb.y*(1.0-vd.y));
//        lineglow += smoothstep(0.4, 1.0, (1.0-uv.y)*(1.0-uv.x))*(1.0-vb.z*(1.0-vd.z));
//        lineglow += smoothstep(0.4, 1.0, (1.0-uv.y)*     uv.x)*(1.0-vb.w*(1.0-vd.w));
//
//        vec3 linCol = 2.0 * vec3(5.0, 0.6, 0.0);
//        linCol *= (0.5 + 0.5 * occlusion) * 0.5;
//        lin += 3.0 * lineglow * linCol;
//
//        color = color*lin;
//        color += 8.0*linCol*vec3(1.0, 2.0, 3.0)*(1.0-wireframe_w);//*(0.5+1.0*sha);
//        color += 0.1*lineglow*linCol;
//        color *= min(0.1, exp(-0.07*t));
//
//        // blend to black & white
//        vec3 col2 = vec3(1.3)*(0.5+0.5*nor.y)* occlusion * wireframe_w * (0.9+0.1*vvv)*exp(-0.04*t);;
//        float mi = sin(-1.57+0.5 * 1.0/*v_Time*/);
//        mi = smoothstep(0.70, 0.75, mi);
//        color = mix(color, col2, mi);
//    }
//    // gamma
//    color = pow(color, vec3(0.45));
//
//    return color;
//}
//
//vec3 path(float t, float ya) {
//    vec2 p  = 100.0*sin(0.02*t*vec2(1.0, 1.2) + vec2(0.1, 0.9));
//    p +=  50.0*sin(0.04*t*vec2(1.3, 1.0) + vec2(1.0, 4.5));
//
//    return vec3(p.x, 18.0 + ya*4.0*sin(0.05*t), p.y);
//}
//
//void main() {
//    float time = 0.01 * v_Time;
//
//    //    // inputs
//    //    vec2 q = v_TexCoordinate;
//    //    vec2 p = -1.0 + 2.0 * q;
//    //    p.x *= v_ScreenSize.x / v_ScreenSize.y;
//    //
//    //    float time = 2.0 * v_Time;
//    //    // camera
//    //    float cr = 0.2 * cos(0.1 * v_Time);
//    vec3 rayOrigin = path(time + 0.0, 1.0);
//    //    vec3 ta = path(time+5.0, 1.0) - vec3(0.0, 6.0, 0.0);
//    //    gro = rayOrigin;
//    vec2 q = v_TexCoordinate;
//    vec2 p = -1.0 + 2.0 * q;
//    p.x *= v_ScreenSize.x/ v_ScreenSize.y;
//    vec3 rayDir = normalize(/*cam * */vec3(p.xy, -2.5));
//
//    vec3 color = render(rayOrigin, rayDir);
//    //    vec3 color = render(v_Position, v_Normal);
//    gl_FragColor = vec4(color, 1.0);
//}